@@ -9,13 +9,13 @@
 set -e
 
 # Compat tests
-if [ "$BASH_VERSION" \< "2.01.1" ] ; then
-	echo "$0: Needs Bash >= 2.01.1" >&2
+if [ "$BASH_VERSION" \< "2.04" ] ; then
+	echo "$0: Needs Bash >= 2.04" >&2
 	exit 1
 fi
 
 export TEXTDOMAIN=bug
-YESNO=$"yYnN"
+export YESNO=$"yYnN"
 
 if [ -z "$EMAIL" ] ; then
 	if  [ ! -r /etc/mailname ] ; then
@@ -34,13 +34,16 @@ if [ "$(type -t realpath 2> /dev/null)" = "" ]; then REALPATH=realp; else REALPA
 # Wait for a keypress and put it in $KEY
 getkey()
 {
-	stty -icanon min 1 || true 2> /dev/null
-	KEY=$(dd bs=1 count=1 2> /dev/null)
-	stty icanon || true 2> /dev/null
-	KEY="${KEY:0:1}"
+	read -r -n 1 KEY
+#	stty -icanon min 1 || true 2> /dev/null
+#	KEY=$(dd bs=1 count=1 2> /dev/null)
+#	stty icanon || true 2> /dev/null
+#	KEY="${KEY:0:1}"
 	echo
 }
 
+export -f getkey
+
 # Usage: yesno <prompt> "yep"|"nop" (<- default)
 #	output: REPLY
 yesno()
@@ -70,6 +73,8 @@ yesno()
 	done
 }
 
+export -f yesno
+
 realp()
 {
 	fname=${1%/}	# strips trailing '/'
@@ -89,6 +94,18 @@ realp()
 	echo $fname
 }
 
+get_stacktrace()
+{
+	local commands="$(tempfile -pbug)"
+	cat << EOF > "$commands"
+set width 180
+set height 2000
+bt
+quit
+EOF
+	gdb -silent -n -x "$commands" -c "$2" "$1" | grep '^#' | tail +2
+}
+
 createfile()
 {
 	for i in $1 $1.{1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20}; do
@@ -100,6 +117,31 @@ createfile()
 	exit 1
 }
 
+findcustomizefiles()
+{
+	if [ -r "/usr/share/bug/$PACKAGE/control" ]; then
+		CONTROLFILE="/usr/share/bug/$PACKAGE/control"
+	elif [ -r "/usr/share/bug/default/$PACKAGE/control" ]; then
+		CONTROLFILE="/usr/share/bug/default/$PACKAGE/control"
+	fi
+
+	if [ -r "/usr/share/bug/$PACKAGE/presubj" ]; then
+		PRESUBJ="/usr/share/bug/$PACKAGE/presubj"
+	elif [ -r "/usr/share/bug/default/$PACKAGE/presubj" ]; then
+		PRESUBJ="/usr/share/bug/default/$PACKAGE/presubj"
+	fi
+
+	if [ -x "/usr/share/bug/$PACKAGE" -a -f "/usr/share/bug/$PACKAGE" ] ; then
+		PACKAGESCRIPT="/usr/share/bug/$PACKAGE"
+	elif [ -d "/usr/share/bug/$PACKAGE" -a -x "/usr/share/bug/$PACKAGE/script" ] ; then
+		PACKAGESCRIPT="/usr/share/bug/$PACKAGE/script"
+	elif [ -x "/usr/share/bug/default/$PACKAGE" -a -f "/usr/share/bug/default/$PACKAGE" ] ; then
+		PACKAGESCRIPT="/usr/share/bug/default/$PACKAGE"
+	elif [ -d "/usr/share/bug/default/$PACKAGE" -a -x "/usr/share/bug/default/$PACKAGE/script" ] ; then
+		PACKAGESCRIPT="/usr/share/bug/default/$PACKAGE/script"
+	fi
+}
+
 sendhelp()
 {
 echo "bug v@VER@"
@@ -111,8 +153,10 @@ Options:
                 report.
     -d          Debug.  Don't send the mail to the bug tracking sys
                 tem, send it to the current user instead.
-    -f          <packagename> is a file belonging to the package
+    -f          <packagename>  is  a file belonging to the  package
                 and bug will search for it.
+    -g          General bug. The bug isn't specific to this system.
+                System information won't be included in the report.
     -L          When used with -f prevents bug from following
                 symlinks.
     -H special_header
@@ -127,7 +171,7 @@ Options:
     -s text     Set the Subject of the bug  report.  Otherwise  bug
                 will ask for a short description of the problem.
     -S severity Set the severity level of the bug report.
-    -x          Do not send a CC to the submitter of the bugreport
+    -x          Do not send a CC to the submitter of the bugreport.
     -z          Do not compress the configfiles by  removing  empty
                 lines and comments.
     -h          Shows this text.
@@ -153,11 +197,12 @@ if grep -q "^$(id -un):" /usr/share/base-passwd/passwd.master ; then
 	exit 1
 fi
 
-while getopts "cdfhH:Lmpqs:S:vzx-" OPT; do
+while getopts "cdfghH:Lmpqs:S:vzx-" OPT; do
 	case $OPT in
 		c)	NOCONFIG=1;;
 		d)	DEBUG=1;;
 		f)	SEARCH=1;;
+		g)	GENERAL=1 ; NOCONFIG=1 ; NODEPS=1 ;;
 		h)	sendhelp $0;;
 		H)	CUSTOM_HEADER[${#CUSTOM_HEADER[*]}]=$OPTARG;;
 		L)	NODEREFERENCESYMLINKS=1;;
@@ -194,6 +239,13 @@ if [ "$SEARCH" ]; then
 		exit 1
 	fi
 	FILE="$PACKAGE"
+	if [ -e "$FILE" ]; then
+		x=$(file "$FILE" | sed -ne "s/^.*core file of '\([^']*\)'.*$/\1/p")
+		if [ "$x" ]; then
+			coredump="$FILE"
+			FILE="$x"
+		fi
+	fi
 	if [ ! -e "$FILE" ]; then
 		OLDIFS="$IFS"
 		IFS=":$IFS"
@@ -215,6 +267,11 @@ if [ "$SEARCH" ]; then
 	if [ ! "$NODEREFERENCESYMLINKS" ]; then
 		FILE=$($REALPATH "$FILE")
 	fi
+	# if it's a #!/bin/* executable, then it isn't an
+	# interesting coredump
+	if head -1 "$FILE" | grep -q '^#' ; then
+		coredump=""
+	fi
 	test "$PRINT" || echo -n $"Searching for package containing ${FILE}..."
 	if [ "$FILE" = "" ]; then
 		echo
@@ -244,21 +301,20 @@ if [ "$PACKAGE" = "" ]; then
 	echo $"Please state in which package you have found a problem, or...
 type one of these bug categories:
 
-base             General bugs in the base system
-boot-floppies    Bugs in the installation discs
-bugs.debian.org  The bug tracking system, @bugs.debian.org
-ftp.debian.org   Problems with the FTP site
-nonus.debian.org Problems with the non-us FTP site
-www.debian.org   Problems with the WWW site
-manual           Bugs in the manual
-project          Problems related to Project administration
-general          General problems (e.g., that many manpages are mode 755)
+project          Problems related to Debian Project administration
+general          General problems (e.g. \"many manpages are mode 755\")
+boot-floppies    Bugs in the installation system
 kernel           Problems with the Linux kernel, or that shipped with Debian
-lists.debian.org The mailing lists debian-*@lists.debian.org."
+base             General bugs in the base system (baseX_Y.tgz)
+bugs.debian.org  Problems with the bug tracking system itself
+ftp.debian.org   Problems with the FTP site
+nonus.debian.org Problems with the non-US FTP site
+www.debian.org   Problems with the WWW site
+lists.debian.org The mailing lists (debian-*@lists.debian.org)"
 	echo
-	read -p $"package? " -er PACKAGE
+	read -p $"Package? " -er PACKAGE
 	if [ "$PACKAGE" = "" ]; then
-		echo $"Canceled!"
+		echo $"Cancelled!"
 		exit 0
 	fi
 fi
@@ -283,6 +339,28 @@ while [ $i -gt 0 -o $i -eq 0 ]; do
 	i=$(($i - 1))
 done
 
+# Now the package name is selected and validated.
+
+findcustomizefiles
+
+# $1 = variable to set
+# $2 = header name
+get_conf_item()
+{
+	local val
+	local key=$2
+	if [ -n "$CONTROLFILE" ]; then
+		if val=$( grep -i "$key[[:space:]]*:" $CONTROLFILE ) ; then
+			val=$(echo "$val" | sed -e "s/^[[:space:]]*$key[[:space:]]*:[[:space:]]\([^[:space:]].*[^[:space:]]\)[[:space:]]*$/\1/i"  )
+			if [ -n "$val" ]; then
+				eval "$1=\"$val\""
+			fi
+		fi
+	fi
+}
+
+get_conf_item submitas submit-as
+get_conf_item bts send-to
 
 #shift || true
 
@@ -293,7 +371,7 @@ if [ "$EDITOR" = "" ]; then
 			EDITOR="$($REALPATH /usr/bin/editor)"
 		else
 			if [ ! -x /bin/ae ]; then
-				echo $"$0: Cannot locate a texteditor." >&2
+				echo $"$0: Cannot locate a text editor." >&2
 				exit 1
 			fi
 			EDITOR="ae"
@@ -341,6 +419,11 @@ setsubject()
 #				--inputbox "Please describe your problems using $PACKAGE" \
 #				8 78 2> /tmp/bugtmp...
 #			SUBJECT=`cat /tmp/bugtmp...`
+			if [ -n "$PRESUBJ" ]; then
+				echo
+				cat "$PRESUBJ"
+				echo
+			fi
 			echo $"Please describe your problems using $PACKAGE in one line."
 			read -er SUBJECT ;
 		done
@@ -368,9 +451,8 @@ setseverity()
        in some systems, or fails to comply current policy documents. Most
        bugs are in this category.
 
-       2) This bug makes the package unsuitable for a stable Debian release.
-       The package should be removed from a stable distribution if this
-       bug is not fixed.
+       2) Important Bug. Has a major effect on the usability of a package,
+       without rendering it completely unusable to everyone.
 
        3) Dangerous bug. Makes the package in question unusable by anyone or
        mostly so, or causes data loss, or introduces a security hole allowing
@@ -403,20 +485,34 @@ setseverity()
 template()
 {
 	if [ "$SUBJECT" ]; then
-		echo "Subject: $SUBJECT"
+		echo "Subject: $SUBJECT" >&3
 	fi
- 	cat <<-EOF
-		Package: $PACKAGE
+ 	cat <<-EOF >&3
+		Package: ${submitas:-$PACKAGE}
 		Version: $VERSION
 		Severity: $SEVERITY
 
-		$*
-
-		-- System Information
-		Debian Release: `cat /etc/debian_version`
-		Kernel Version: `uname -a`
-
 	EOF
+
+	if [ -n "$PACKAGESCRIPT" ]; then
+		scriptout=$(tempfile -pbug)
+		$PACKAGESCRIPT
+		echo >&3
+	fi
+	
+	if [ -n "$*" ]; then
+		echo "$*" >&3
+	fi
+
+	if [ -z "$GENERAL" ]; then
+	 	cat <<-EOF >&3
+	
+			-- System Information
+			Debian Release: `cat /etc/debian_version`
+			Kernel Version: `uname -a`
+		
+		EOF
+	fi
 }
 
 trap 'echo $"$0 interrupted." ; rm -f $TEMPS ; exit 0' SIGINT
@@ -437,9 +533,10 @@ if [ -f /var/lib/dpkg/info/$PACKAGE.list ]; then
 	test "$PRINT" || echo $"done."
 	setsubject
 	setseverity
-	template "$*" >$TEMP1
+	>&3
+	template "$*" 3> $TEMP1
 
-	if [ "$DEPS" != "" ]; then
+	if [ -z "$NODEPS" -a -n "$DEPS" ]; then
 		test "$PRINT" || echo -n $"Checking ${PACKAGE}'s dependencies..."
 		echo "Versions of the packages $PACKAGE depends on:" >>$TEMP1
 		DEPS=`echo "$DEPS"|tr "|,\n" "  "|sed -e "s/ ([^)]*)//g" -e "s/Pre-Depends://g" -e "s/Depends://g" -e "s/  / /g" `
@@ -479,33 +576,62 @@ if [ -f /var/lib/dpkg/info/$PACKAGE.list ]; then
 		test "$PRINT" || echo $"done."
 	fi
 
-	if [ "$MODCONF" != "" ]; then
+	if test "$coredump" ; then
+		if type gdb &> /dev/null ; then
+			test "$PRINT" || echo -n $"Including stack trace from coredump..."
+			echo >> $TEMP1
+			echo "--- Stack trace:" >> $TEMP1
+			get_stacktrace "$FILE" "$coredump" >> $TEMP1
+			test "$PRINT" || echo $"done."
+		else
+			echo $"I can't include a stack trace in the bug report without gdb installed."
+			sleep 3
+		fi
+	fi
+
+	if [ -n "$MODCONF" ]; then
 		test "$PRINT" || echo -n $"Including modified configuration files..."
+		declare -i conftotal=0 confparcial
 		for i in $MODCONF; do
 			if ls -lL "$i" | grep -qv ^.......r ; then
+				echo -e "\n--- Ignoring conffile $i (not world readable)" >>$TEMP1
 				continue
 			fi
-			echo -e "\n--- Begin $i (modified conffile)" >>$TEMP1
-			if [ -r $i ]; then
+			TEMPCONF=$(tempfile -pbug)
+			echo -e "\n--- Begin $i (modified conffile)" >$TEMPCONF
+			if [ -r "$i" ]; then
 				if file -L $i | grep -q "text"; then
-					if [ "$NOZAP" ]; then
-						cp $i $TEMP1
+					if [ -n "$NOZAP" ]; then
+						cat $i >> $TEMPCONF
 					else
-						sed -e '/^#[^!]/d' -e '/^$/d' <$i >>$TEMP1
+						sed -e '/^#[^!]/d' -e '/^#*$/d' <$i >>$TEMPCONF
 					fi
 				else
-					echo "*** Contents not text ***" >>$TEMP1
+					echo "*** Contents not text ***" >>$TEMPCONF
 				fi
 			else
-				echo "Config file not present or no permissions for access" >>$TEMP1
+				echo "Config file not present or no permissions for access" >>$TEMPCONF
 			fi
-			echo -e "\n--- End $i" >>$TEMP1
+			echo -e "\n--- End $i" >>$TEMPCONF
+			confparcial=$(find "$TEMPCONF" -printf %s)
+			if [ $confparcial -gt 8192 ]; then
+				echo -e "\n--- Ignoring modified conffile $i (>8k)" >>$TEMP1
+				continue
+			fi
+			conftotal=$(( $conftotal + $confparcial ))
+			if [ $conftotal -gt 16384 ]; then
+				echo -e "\n--- Ignoring modified conffile $i (already included 16k of conffiles)" >>$TEMP1
+				continue
+			fi
+			cat "$TEMPCONF" >> $TEMP1
 		done
 		test "$PRINT" || echo $"done."
 	fi
 else
 	VERSION="N/A"
-	for i in base bootdisk rootdisk bugs.debian.org ftp.debian.org www.debian.org manual project general kernel lists.debian.org ; do
+	for i in base boot-floppies bugs.debian.org ftp.debian.org\
+		www.debian.org project general kernel lists.debian.org \
+		nonus.debian.org ; do
 		if [ "$PACKAGE" = "$i" ]; then
 			PSEUDO=yep
 			VERSION=$(date -u +"%Y%m%d")
@@ -521,7 +647,8 @@ else
 	fi
 	setsubject
 	setseverity
-	template "$*" >$TEMP1
+	>&3
+	template "$*" 3>$TEMP1
 fi
 
 if [ -z "$1" -a -z "$PRINT" ]; then
@@ -564,7 +691,7 @@ if [ -z "$1" -a -z "$PRINT" ]; then
 			fi
 
 			while true; do
-				echo -n $"Do you really want to submit this bug report? [y|n|E=edit] "
+				echo -n $"Do you really want to submit this bug report? [y|n|E=edit|I=spell check] "
 
 				getkey
 
@@ -581,8 +708,19 @@ if [ -z "$1" -a -z "$PRINT" ]; then
 					do_edit=nop
 					break
 				fi
+				
+				# if 'i'
+				if [ "$KEY" = i -o "$KEY" = I ]; then
+					if [ ! -x /usr/bin/ispell -o ! -r /usr/lib/ispell/american.hash ]; then
+						echo
+						echo $"Ispell and USA English wordlists must be installed for this feature to work!"
+						echo
+					else
+						ispell -d american $TEMP1
+					fi
+				fi
 
-				# if \n
+				# if \n | e
 				if [ "$KEY" = "" -o "$KEY" = $"e" \
 						-o "$KEY" = $"E" ]; then
 					REPLY=edit
@@ -595,7 +733,7 @@ if [ -z "$1" -a -z "$PRINT" ]; then
 		if [ $REPLY = nop ]; then
 			SAVEFILE=$(createfile /var/tmp/bug.dead )
 			cat "$TEMP1" >> $SAVEFILE
-			echo $"Bug report cancelled (report text left in $SAVEFILE)."
+			echo $"Bug report canceled (report text left in $SAVEFILE)."
 			exit 0
 		fi
 		rm $TEMPBACKUP
@@ -615,14 +753,14 @@ if [ "$PRINT" ]; then
 	exit 0
 fi
 
-MADDR="submit@bugs.debian.org"
+MADDR="submit@${bts:-bugs.debian.org}"
 
 if [ "$QUIET" ]; then
-	MADDR="quiet@bugs.debian.org"
+	MADDR="quiet@${bts:-bugs.debian.org}"
 fi
 
 if [ "$MAINT" ]; then
-	MADDR="maintonly@bugs.debian.org"
+	MADDR="maintonly@${bts:-bugs.debian.org}"
 fi
 
 if [ "$DEBUG" ]; then
@@ -646,11 +784,7 @@ fi
 		i=$((i+1))
 	done
 	echo
-	if [ -z "$1" ]; then
-		sed <$TEMP1 -e "1d"
-	else
-		echo $@
-	fi
+	sed <$TEMP1 -e "1d"
 } |
 {
 	{
